//
// Cforall Version 1.0.0 Copyright (C) 2015 University of Waterloo
//
// The contents of this file are covered under the licence agreement in the
// file "LICENCE" distributed with Cforall.
//
// Cost.h --
//
// Author           : Richard C. Bilson
// Created On       : Sun May 17 09:39:50 2015
// Last Modified By : Rob Schluntz
// Last Modified On : Wed Jul 22 16:43:10 2015
// Update Count     : 4
//

#ifndef COST_H
#define COST_H

#include <iostream>

namespace ResolvExpr {
	class Cost {
	  public:
		Cost();
		Cost( int unsafeCost, int polyCost, int safeCost );

		void incUnsafe( int inc = 1 );
		void incPoly( int inc = 1 );
		void incSafe( int inc = 1 );

		Cost operator+( const Cost &other ) const;
		Cost operator-( const Cost &other ) const;
		Cost &operator+=( const Cost &other );
		bool operator<( const Cost &other ) const;
		bool operator==( const Cost &other ) const;
		bool operator!=( const Cost &other ) const;
		friend std::ostream &operator<<( std::ostream &os, const Cost &cost );

		static const Cost zero;
		static const Cost infinity;
		static const Cost unsafe;
		static const Cost poly;
		static const Cost safe;
	  private:
		int compare( const Cost &other ) const;

		int unsafeCost;
		int polyCost;
		int safeCost;
	};

	inline Cost::Cost() : unsafeCost( 0 ), polyCost( 0 ), safeCost( 0 ) {}

	inline Cost::Cost( int unsafeCost, int polyCost, int safeCost ) : unsafeCost( unsafeCost ), polyCost( polyCost ), safeCost( safeCost ) {}

	inline void Cost::incUnsafe( int inc ) {
		unsafeCost += inc;
	}

	inline void Cost::incPoly( int inc ) {
		polyCost += inc;
	}

	inline void Cost::incSafe( int inc ) {
		safeCost += inc;
	}

	inline Cost Cost::operator+( const Cost &other ) const {
		return Cost( unsafeCost + other.unsafeCost, polyCost + other.polyCost, safeCost + other.safeCost );
	}

	inline Cost Cost::operator-( const Cost &other ) const {
		return Cost( unsafeCost - other.unsafeCost, polyCost - other.polyCost, safeCost - other.safeCost );
	}

	inline Cost &Cost::operator+=( const Cost &other ) {
		unsafeCost += other.unsafeCost;
		polyCost += other.polyCost;
		safeCost += other.safeCost;
		return *this;
	}

	inline bool Cost::operator<( const Cost &other ) const {
		if ( *this == infinity ) return false;
		if ( other == infinity ) return true;

		if ( unsafeCost > other.unsafeCost ) {
			return false;
		} else if ( unsafeCost < other.unsafeCost ) {
			return true;
		} else if ( polyCost > other.polyCost ) {
			return false;
		} else if ( polyCost < other.polyCost ) {
			return true;
		} else if ( safeCost > other.safeCost ) {
			return false;
		} else if ( safeCost < other.safeCost ) {
			return true;
		} else {
			return false;
		} // if
	}

	inline bool Cost::operator==( const Cost &other ) const {
		return unsafeCost == other.unsafeCost
			&& polyCost == other.polyCost
			&& safeCost == other.safeCost;
	}

	inline bool Cost::operator!=( const Cost &other ) const {
		return !( *this == other );
	}

	inline std::ostream &operator<<( std::ostream &os, const Cost &cost ) {
		os << "( " << cost.unsafeCost << ", " << cost.polyCost << ", " << cost.safeCost << " )";
		return os;
	}
} // namespace ResolvExpr

#endif // COST_H

// Local Variables: //
// tab-width: 4 //
// mode: c++ //
// compile-command: "make install" //
// End: //
